@@ -1,76 +1,74 @@
+import warnings
 import kubernetes.client as k8s
 
-from typing import List, Optional, Union
+from typing import List, Optional, Union, Dict
+from airflow_kubernetes_job_operator.exceptions import KubernetesJobOperatorException
+from airflow_kubernetes_job_operator.kube_api.collections import KubeResourceDescriptor
 from airflow_kubernetes_job_operator.job_runner import JobRunnerDeletePolicy
-from airflow_kubernetes_job_operator.utils import resolve_relative_path
+from airflow_kubernetes_job_operator.utils import resolve_relative_path, dict_merge, dict_remove_empty_cols
 from airflow_kubernetes_job_operator.kubernetes_job_operator import KubernetesJobOperator, xcom_value_parser
-from airflow_kubernetes_job_operator.config import DEFAULT_VALIDATE_BODY_ON_INIT, IS_AIRFLOW_ONE
-
-
-LOAD_CLASSES_FROM_KUBERNETES_CLIENT = True
-try:
-    if IS_AIRFLOW_ONE:
-        from airflow.contrib.kubernetes.kubernetes_request_factory import pod_request_factory
-        from airflow.contrib.kubernetes import pod_generator
-        from airflow.contrib.kubernetes.pod import Resources
-        from airflow.contrib.kubernetes.volume_mount import VolumeMount
-        from airflow.contrib.kubernetes.volume import Volume
-        from airflow.contrib.kubernetes.secret import Secret
-
-        LOAD_CLASSES_FROM_KUBERNETES_CLIENT = False
-except Exception:
-    pass
-
-if LOAD_CLASSES_FROM_KUBERNETES_CLIENT:
-    from kubernetes.client import (
-        V1ResourceRequirements as Resources,
-        V1Volume as Volume,
-        V1VolumeMount as VolumeMount,
-        V1Secret as Secret,
-    )
-
-    # Back support for old kubernetes client.
-    pod_generator = None
-    pod_request_factory = None
+from airflow_kubernetes_job_operator.config import (
+    DEFAULT_VALIDATE_BODY_ON_INIT,
+    DEFAULT_EXECUTION_OBJECT_PATHS,
+    KubernetesJobOperatorDefaultExecutionResource,
+)
+from airflow_kubernetes_job_operator.kubernetes_legacy_pod_generators import (
+    create_legacy_kubernetes_pod,
+    Volume,
+    Port,
+    VolumeMount,
+    Secret,
+    Resources,
+    Affinity,
+    PodRuntimeInfoEnv,
+)
 
 
 class KubernetesLegacyJobOperator(KubernetesJobOperator):
     def __init__(
         self,
-        namespace: str = None,
-        image: str = None,
-        cmds: List[str] = None,
-        arguments: List[str] = None,
-        ports: list = None,
-        volume_mounts: List[VolumeMount] = None,
-        volumes: List[Volume] = None,
-        env_vars: dict = None,
-        secrets: List[Secret] = None,
-        in_cluster: bool = None,
-        cluster_context: str = None,
-        labels: dict = None,
-        startup_timeout_seconds: float = 120,
+        *args,
+        namespace: Optional[str] = None,
+        image: Optional[str] = None,
+        name: Optional[str] = None,
+        cmds: Optional[List[str]] = None,
+        arguments: Optional[List[str]] = None,
+        ports: Optional[List[Port]] = None,
+        volume_mounts: Optional[List[VolumeMount]] = None,
+        volumes: Optional[List[Volume]] = None,
+        env_vars: Optional[Union[dict, List[k8s.V1EnvVar]]] = None,
+        env_from: Optional[List[k8s.V1EnvFromSource]] = None,
+        secrets: Optional[List[Secret]] = None,
+        in_cluster: Optional[bool] = None,
+        cluster_context: Optional[str] = None,
+        labels: Optional[Dict] = None,
+        reattach_on_restart: bool = True,
+        startup_timeout_seconds: int = 120,
         get_logs: bool = True,
-        image_pull_policy: str = "IfNotPresent",
-        annotations: dict = None,
-        resources=None,
-        affinity: dict = None,
-        config_file: str = None,
-        node_selectors: dict = None,
-        image_pull_secrets: str = None,
-        service_account_name: str = "default",
+        image_pull_policy: Optional[str] = None,
+        annotations: Optional[Dict] = None,
+        resources: Optional[Resources] = None,
+        affinity: Optional[Affinity] = None,
+        config_file: Optional[str] = None,
+        node_selectors: Optional[dict] = None,
+        node_selector: Optional[dict] = None,
+        image_pull_secrets: Optional[Union[List[k8s.V1LocalObjectReference], str]] = None,
+        service_account_name: Optional[str] = None,
         is_delete_operator_pod: bool = False,
         hostnetwork: bool = False,
-        tolerations: List[dict] = None,
-        configmaps: List[str] = None,
-        security_context: dict = None,
-        pod_runtime_info_envs: dict = None,
-        dnspolicy: str = None,
-        # new args.
+        tolerations: Optional[List[k8s.V1Toleration]] = None,
+        security_context: Optional[Dict] = None,
+        dnspolicy: Optional[str] = None,
+        schedulername: Optional[str] = None,
+        full_pod_spec: Optional[k8s.V1Pod] = None,
         init_containers: Optional[List[k8s.V1Container]] = None,
-        env_from: List[str] = None,
-        schedulername: str = None,
-        priority_class_name: str = None,
+        log_events_on_failure: bool = False,
+        do_xcom_push: bool = False,
+        pod_template_file: Optional[str] = None,
+        priority_class_name: Optional[str] = None,
+        pod_runtime_info_envs: List[PodRuntimeInfoEnv] = None,
+        termination_grace_period: Optional[int] = None,
+        configmaps: Optional[str] = None,
         # job operator args
         body: str = None,
         body_filepath: str = None,
@@ -80,7 +78,6 @@ class KubernetesLegacyJobOperator(KubernetesJobOperator):
         jinja_job_args: dict = None,
         on_kube_api_event: callable = None,
         parse_xcom_event: xcom_value_parser = xcom_value_parser,
-        *args,
         **kwargs,
     ):
         """
@@ -98,6 +95,8 @@ class KubernetesLegacyJobOperator(KubernetesJobOperator):
         :type image: str
         :param namespace: the namespace to run within kubernetes
         :type namespace: str
+        :param name: The name prefix for the executing pod. (default: task_id)
+        :type name: str
         :param cmds: entrypoint of the container. (templated)
             The docker images's entrypoint is used if this is not provide.
         :type cmds: list[str]
@@ -176,7 +175,8 @@ class KubernetesLegacyJobOperator(KubernetesJobOperator):
                 (default: {from env/airflow config: AIRFLOW__KUBE_JOB_OPERATOR__validate_body_on_init or False})
             jinja_job_args {dict} -- A dictionary or object to be used in the jinja template to render
                 arguments. The jinja args are loaded under the keyword "job".
-            on_kube_api_event {callable, optional} -- a method to catch api events when called. lambda api_event, context: ...
+            on_kube_api_event {callable, optional} -- a method to catch api events when called. lambda api_event,
+                context: ...
             parse_xcom_event {xcom_value_parser, optional} -- parse an incoming xcom event value.
                 Must return a dictionary with key/value pairs.
         """
@@ -186,14 +186,19 @@ class KubernetesLegacyJobOperator(KubernetesJobOperator):
             else JobRunnerDeletePolicy.Never
         )
 
+        body_filepath = body_filepath or pod_template_file
+
         if body_filepath is not None:
             body_filepath = resolve_relative_path(body_filepath, 2)
 
+        if full_pod_spec is not None and body_filepath is None:
+            body_filepath = DEFAULT_EXECUTION_OBJECT_PATHS[KubernetesJobOperatorDefaultExecutionResource.Pod]
+
         super().__init__(
             command=cmds or [],
             arguments=arguments or [],
-            envs=env_vars or {},
             image=image,
+            name_prefix=name,
             namespace=namespace,
             body=body,
             body_filepath=body_filepath,
@@ -213,13 +218,26 @@ class KubernetesLegacyJobOperator(KubernetesJobOperator):
             **kwargs,
         )
 
+        assert image is not None, "You must provide an image"
+
+        # If true, loaded from yaml and should not override spec.
+        self.loaded_from_full_spec = full_pod_spec is not None or body is not None or body_filepath is not None
+
         # adding self properties.
         self.labels = labels or {}
         self.ports = ports or []
         self.volume_mounts = volume_mounts or []
         self.volumes = volumes or []
         self.secrets = secrets or []
-        self.node_selectors = node_selectors or {}
+        if node_selectors:
+            # Node selectors is incorrect based on k8s API
+            warnings.warn("node_selectors is deprecated. Please use node_selector instead.", DeprecationWarning)
+            self.node_selector = node_selectors
+        elif node_selector:
+            self.node_selector = node_selector
+        else:
+            self.node_selector = {}
+
         self.annotations = annotations or {}
         self.affinity = affinity or {}
         self.resources = self._set_resources(resources)
@@ -232,8 +250,18 @@ class KubernetesLegacyJobOperator(KubernetesJobOperator):
         self.pod_runtime_info_envs = pod_runtime_info_envs or []
         self.dnspolicy = dnspolicy
 
+        # new values.
+        self.full_pod_spec = full_pod_spec
+        self.termination_grace_period = termination_grace_period
+
+        # Value is ignored. Xcoms are always active on KJO (using logging)
+        self._ignored_log_events_on_failure = log_events_on_failure
+        self._ignored_do_xcom_push = do_xcom_push
+        self._unsupported_reattach_on_restart = reattach_on_restart
+
         self.init_containers = init_containers
         self.env_from = env_from
+        self.env_vars = env_vars
         self.schedulername = schedulername
         self.priority_class_name = priority_class_name
 
@@ -247,102 +275,38 @@ class KubernetesLegacyJobOperator(KubernetesJobOperator):
 
     def prepare_and_update_body(self):
         # call to prepare the raw body.
+        pod = KubeResourceDescriptor(
+            create_legacy_kubernetes_pod(self),
+        )
+
+        if "name" in pod.metadata:
+            self.name_prefix = self.name_prefix or pod.metadata["name"]
+            del pod.metadata["name"]
+
+        dict_remove_empty_cols(pod.body)
+
+        pod.body["metadata"] = pod.body.get("metadata", {})
+        pod.body["spec"] = pod.body.get("spec", {})
+
         super().prepare_and_update_body()
 
-        pod_body = None
-        if pod_generator is not None:
-            # old pod generator
-            gen = pod_generator.PodGenerator()
-
-            for port in self.ports:
-                gen.add_port(port)
-            for mount in self.volume_mounts:
-                gen.add_mount(mount)
-            for volume in self.volumes:
-                gen.add_volume(volume)
-
-            job_obj = self.job_runner.body[0]
-
-            # selecting appropriate pod values.
-            all_labels = {}
-            all_labels.update(self.labels)
-            all_labels.update(job_obj["spec"]["template"]["metadata"].get("labels", {}))
-            image = self.image or job_obj["spec"]["template"]["spec"]["containers"][0].get("image", None)
-            cmds = self.command or job_obj["spec"]["template"]["spec"]["containers"][0].get("command", [])
-            arguments = self.arguments or job_obj["spec"]["template"]["spec"]["containers"][0].get("args", [])
-
-            pod = gen.make_pod(
-                namespace=job_obj["metadata"]["namespace"],
-                image=image,
-                pod_id=job_obj["metadata"]["name"],
-                cmds=cmds,
-                arguments=arguments,
-                labels=all_labels,
-            )
-
-            pod.service_account_name = self.service_account_name
-            pod.secrets = self.secrets
-            pod.envs = self.envs
-            pod.image_pull_policy = self.image_pull_policy
-            pod.image_pull_secrets = self.image_pull_secrets
-            pod.annotations = self.annotations
-            pod.resources = self.resources
-            pod.affinity = self.affinity
-            pod.node_selectors = self.node_selectors
-            pod.hostnetwork = self.hostnetwork
-            pod.tolerations = self.tolerations
-            pod.configmaps = self.configmaps
-            pod.security_context = self.security_context
-            pod.pod_runtime_info_envs = self.pod_runtime_info_envs
-            pod.dnspolicy = self.dnspolicy
-
-            # old pod generation.. moving to new one
-            pod_body = pod_request_factory.SimplePodRequestFactory().create(pod)
-        else:
-            pod_body = k8s.V1Pod(
-                api_version="v1",
-                kind="Pod",
-                metadata=k8s.V1ObjectMeta(
-                    namespace=self.namespace,
-                    labels=self.labels,
-                    name="legacy",
-                    annotations=self.annotations,
-                ),
-                spec=k8s.V1PodSpec(
-                    node_selector=self.node_selectors,
-                    affinity=self.affinity,
-                    tolerations=self.tolerations,
-                    init_containers=self.init_containers,
-                    containers=[
-                        k8s.V1Container(
-                            image=self.image,
-                            name="main",
-                            command=self.command,
-                            ports=self.ports,
-                            resources=self.resources,
-                            volume_mounts=self.volume_mounts,
-                            args=self.arguments,
-                            env=None if self.envs is None else self._get_kubernetes_env_list(),
-                            env_from=self.env_from,
-                        )
-                    ],
-                    image_pull_secrets=self.image_pull_secrets,
-                    service_account_name=self.service_account_name,
-                    host_network=self.hostnetwork,
-                    security_context=self.security_context,
-                    dns_policy=self.dnspolicy,
-                    scheduler_name=self.schedulername,
-                    restart_policy="Never",
-                    priority_class_name=self.priority_class_name,
-                    volumes=self.volumes,
-                ),
-            )
-            pod_body = {
-                "metadata": self.job_runner.client.api_client.sanitize_for_serialization(pod_body.metadata),
-                "spec": self.job_runner.client.api_client.sanitize_for_serialization(pod_body.spec),
+        if len(self.job_runner.body) == 0:
+            self.job_runner.body[0] = {
+                "apiVersion": "batch/v1",
+                "kind": "Pod",
+                "metadata": {},
+                "spec": {},
             }
 
-        # reset the name
-        del pod_body["metadata"]["name"]
-        pod_body["metadata"].update(self.job_runner.body[0]["spec"]["template"]["metadata"])
-        self.job_runner.body[0]["spec"]["template"] = pod_body
+        run_template = KubeResourceDescriptor(self.job_runner.body[0])
+        if run_template.kind_name == "pod":
+            dict_merge(self.job_runner.body[0], pod.body)
+        elif run_template.kind_name == "job":
+            self.job_runner.body[0]["spec"]["template"] = {
+                "metadata": pod.body.get("metadata"),
+                "spec": pod.body.get("spec"),
+            }
+        else:
+            raise KubernetesJobOperatorException(
+                "Invalid kind in operator template. Expected first element run kind to either be Job or Pod"
+            )
